﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace AnswerForMe.Presentation
{
    class BrowserManager
    {
        private string browsingDatabase;
        private TreeView tvwBrowser;
        private BrowserNode[] rootNodes;

        private static readonly string ClosedImage = "CLOSED";
        private static readonly string OpenedImage = "OPENED";
        private static readonly string LeafImage = "LEAF";

        public BrowserManager(TreeView tvwBrowser) : this(tvwBrowser, BrowserNode.RootNodes) {}
        public BrowserManager(TreeView tvwBrowser, BrowserNode[] rootNodes)
        {
            this.tvwBrowser = tvwBrowser;
            this.rootNodes = rootNodes;
        }

        public string BrowsingDatabase
        {
            get { return this.browsingDatabase; }
            set
            {
                this.browsingDatabase = value;
                this.Initialize();
            }
        }

        public void Initialize()
        {
            if (tvwBrowser == null || rootNodes == null) return;
            BrowserNode.BrowsingDatabase = browsingDatabase;
            //Ugly
            tvwBrowser.BeforeExpand -= new TreeViewCancelEventHandler(tvwBrowser_BeforeExpand);
            tvwBrowser.BeforeExpand += new TreeViewCancelEventHandler(tvwBrowser_BeforeExpand);
            tvwBrowser.AfterExpand -= new TreeViewEventHandler(tvwBrowser_AfterExpand);
            tvwBrowser.AfterExpand += new TreeViewEventHandler(tvwBrowser_AfterExpand);
            tvwBrowser.AfterCollapse -= new TreeViewEventHandler(tvwBrowser_AfterCollapse);
            tvwBrowser.AfterCollapse += new TreeViewEventHandler(tvwBrowser_AfterCollapse);
            tvwBrowser.MouseDown -= new MouseEventHandler(tvwBrowser_MouseDown);
            tvwBrowser.MouseDown += new MouseEventHandler(tvwBrowser_MouseDown);
            tvwBrowser.MouseUp -= new MouseEventHandler(tvwBrowser_MouseUp);
            tvwBrowser.MouseUp += new MouseEventHandler(tvwBrowser_MouseUp);
            TreeNodeCollection nodes = tvwBrowser.Nodes;
            nodes.Clear();
            foreach (BrowserNode dbNode in rootNodes)
            {
                TreeNode node = nodes.Add(dbNode.NodeName);
                node.Tag = dbNode;
                node.ImageKey = ClosedImage;
                node.Nodes.Add("");
            }
        }

        public Action<string> HelpTextProduced;

        private void tvwBrowser_BeforeExpand(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)
        {
            TreeNode node = e.Node;
            if (node.Nodes.Count > 0 && node.FirstNode.Tag is BrowserNode) return;

            BrowserNode dbNode = node.Tag as BrowserNode;
            node.Nodes.Clear();            
            foreach (BrowserNode childDbNode in dbNode.GetChildNodes())
            {
                TreeNode childNode = node.Nodes.Add(childDbNode.NodeName);
                childNode.Tag = childDbNode;
                if (!childDbNode.IsLeafNode)
                {
                    childNode.ImageKey = childNode.SelectedImageKey = ClosedImage;
                    childNode.Nodes.Add("");
                }
                else
                {
                    childNode.ImageKey = childNode.SelectedImageKey = LeafImage;
                }
            }
        }

        private void tvwBrowser_AfterExpand(object sender, System.Windows.Forms.TreeViewEventArgs e)
        {
            TreeNode node = e.Node;
            if (node.Nodes.Count > 0)
                node.ImageKey = node.SelectedImageKey = OpenedImage;
        }

        private void tvwBrowser_AfterCollapse(object sender, System.Windows.Forms.TreeViewEventArgs e)
        {
            TreeNode node = e.Node;
            if (node.Nodes.Count > 0)
                node.ImageKey = node.SelectedImageKey = ClosedImage;
        }

        private void tvwBrowser_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                TreeNode node = tvwBrowser.GetNodeAt(e.X, e.Y);
                if (node != null)
                    tvwBrowser.SelectedNode = node;
            }
        }

        private void tvwBrowser_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right && tvwBrowser.SelectedNode != null)
            {
                if (null == tvwBrowser.SelectedNode) return;
                BrowserNode dbNode = tvwBrowser.SelectedNode.Tag as BrowserNode;
                if (null == dbNode) return;
                if (dbNode.NodeType == BrowserNodeType.UserStoredProc || dbNode.NodeType == BrowserNodeType.SystemStoredProc ||
                    dbNode.NodeType == BrowserNodeType.Function || dbNode.NodeType == BrowserNodeType.View)
                {
                    string text = dbNode.RetrieveHelpText();
                    if (null != HelpTextProduced)
                        HelpTextProduced(text);
                }
            }
        }
    }
}