﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

using AnswerForMe.Model;

namespace AnswerForMe.Presentation
{
    enum BrowserNodeType 
    {
        UserTableList,
        SystemTableList,
        ViewList,
        UserStoredProcList,
        SystemStoredProcList,
        ExtendedProcList,
        FunctionList,
        
        UserTable,
        SystemTable,
        View,
        UserStoredProc,
        SystemStoredProc,
        ExtendedProc,
        Function,

        ColumnList,
        Column, 

        NULLType
    };

    class BrowserNode
    {
        private BrowserNodeType type;
        private string name;
        private string owner;
        private int id = -1;

        public static string BrowsingDatabase
        {
            get;
            set;
        }

        public static BrowserNode UserTableListNode = new BrowserNode()
        {
            type = BrowserNodeType.UserTableList,
            name = "用户表"
        };
        public static BrowserNode SystemTableListNode = new BrowserNode()
        {
            type = BrowserNodeType.SystemTableList,
            name = "系统表"
        };
        public static BrowserNode ViewListNode = new BrowserNode()
        {
            type = BrowserNodeType.ViewList,
            name = "视图"
        };
        public static BrowserNode UserStoredProcListNode = new BrowserNode()
        {
            type = BrowserNodeType.UserStoredProcList,
            name = "用户存储过程"
        };
        public static BrowserNode SystemStoredProcListNode = new BrowserNode()
        {
            type = BrowserNodeType.SystemStoredProcList,
            name = "系统存储过程"
        };
        public static BrowserNode ExtendedProcListNode = new BrowserNode()
        {
            type = BrowserNodeType.ExtendedProcList,
            name = "扩展过程"
        };
        public static BrowserNode FunctionListNode = new BrowserNode()
        {
            type = BrowserNodeType.FunctionList,
            name = "函数"
        };
        public static BrowserNode[] RootNodes = new BrowserNode[] 
        {
            UserTableListNode,
            SystemTableListNode,
            ViewListNode,
            UserStoredProcListNode,
            SystemStoredProcListNode,
            ExtendedProcListNode,
            FunctionListNode
        };

        public BrowserNodeType NodeType
        {
            get { return type; }
        }
        public string NodeName
        {
            get { return (String.IsNullOrEmpty(owner) ? "" : owner + ".") + name; }
        }
        public string NodeOwner
        {
            get { return owner; }
        }

        public string RetrieveHelpText()
        {
            return QueryHelper.RetrieveHelpText(id, BrowsingDatabase);
        }

        public BrowserNode[] GetChildNodes()
        {
            if (Array.IndexOf<BrowserNode>(RootNodes, this) != -1)
                return RetrieveObjectList();
            else if (type == BrowserNodeType.UserTable || type == BrowserNodeType.SystemTable || type == BrowserNodeType.View)
                return new BrowserNode[]{
                    new BrowserNode() {
                        type = BrowserNodeType.ColumnList,
                        name = "列",
                        id = this.id
                    }
                };
            else if (this.type == BrowserNodeType.ColumnList)
                return RetrieveColumnList();
            else
                return null;
        }

        public bool IsLeafNode
        {
            get
            {   return NodeType == BrowserNodeType.UserStoredProc ||
                    NodeType == BrowserNodeType.SystemStoredProc ||
                    NodeType == BrowserNodeType.ExtendedProc ||
                    NodeType == BrowserNodeType.Function ||
                    NodeType == BrowserNodeType.Column;
            }
        }

        private BrowserNode[] RetrieveColumnList()
        {
            DataTable table = QueryHelper.RetrieveColumnList(id, BrowsingDatabase);
            if (table == null) return new BrowserNode[] { };

            List<BrowserNode> list = new List<BrowserNode>();
            //COLUMN_NAME, DATA_TYPE, EXTENSION, IS_NULLABLE
            foreach (DataRow row in table.Rows)
            {
                list.Add(new BrowserNode()
                {
                    type = BrowserNodeType.Column,                   
                    name = row["COLUMN_NAME"].ToString() + " " + "(" +
                        row["DATA_TYPE"].ToString() + (QueryHelper.IsDataTypeFixed(row["DATA_TYPE"].ToString()) ? "" : row["EXTENSION"].ToString()) + "," + " " + 
                        (row["IS_NULLABLE"].ToString().StartsWith("Y") ? "Null" : "Not Null") + ")",
                    id = this.id
                });
            }
            return list.ToArray<BrowserNode>();
        }

        private BrowserNode[] RetrieveObjectList()
        {
            DataTable table = QueryHelper.RetrieveObjectList(name, BrowsingDatabase);
            if (table == null) return new BrowserNode[]{};

            List<BrowserNode> list = new List<BrowserNode>();
            BrowserNodeType childNodeType = ChildNodeType;
            foreach (DataRow row in table.Rows)
            {
                list.Add(new BrowserNode()
                {
                    type = childNodeType,
                    name = row["objname"].ToString(),
                    owner = row["objowner"].ToString(),
                    id = Convert.ToInt32(row["id"])
                });
            }
            return list.ToArray<BrowserNode>();
        }

        private BrowserNodeType ChildNodeType
        {
            get
            {
                switch (type)
                {
                    case BrowserNodeType.UserTableList:
                        return BrowserNodeType.UserTable;
                    case BrowserNodeType.SystemTableList:
                        return BrowserNodeType.SystemTable;
                    case BrowserNodeType.ViewList:
                        return BrowserNodeType.View;
                    case BrowserNodeType.UserStoredProcList:
                        return BrowserNodeType.UserStoredProc;
                    case BrowserNodeType.SystemStoredProcList:
                        return BrowserNodeType.SystemStoredProc;
                    case BrowserNodeType.ExtendedProcList:
                        return BrowserNodeType.ExtendedProc;
                    case BrowserNodeType.FunctionList:
                        return BrowserNodeType.Function;
                    case BrowserNodeType.ColumnList:
                        return BrowserNodeType.Column;
                    default:
                        return BrowserNodeType.NULLType;
                }
            }
        }
    }
}