﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;

using AnswerForMe.Model;
using AnswerForMe.Properties;

namespace AnswerForMe.Presentation
{
    public partial class MainForm : Form
    {
        private IAFMConnection currentConnection;
        private IList<QueryExecutor> queryExecutorList = new List<QueryExecutor>();
        private BrowserManager browerManager;

        private string[] DatabaseList
        {
            set
            {
                cboDatabaseList.Items.Clear();
                cboDatabaseList.Items.AddRange(value);
            }
        }

        public MainForm()
        {
            System.Threading.Thread.CurrentThread.Name = "Main Thread";
            InitializeComponent();
            connectMenuItem.Click += new EventHandler(connectMenuItem_Click);
            newMenuItem.Click += new EventHandler(newMenuItem_Click);
            openMenuItem.Click += new EventHandler(openMenuItem_Click);
            saveMenuItem.Click += new EventHandler(saveMenuItem_Click);
            saveasMenuItem.Click += new EventHandler(saveasMenuItem_Click);
            exitMenuItem.Click += new EventHandler(exitMenuItem_Click);
            executeMenuItem.Click += new EventHandler(executeMenuItem_Click);
            cancelexecuteMenuItem.Click += new EventHandler(cancelexecuteMenuItem_Click);
            intextMenuItem.Click += new EventHandler(intextMenuItem_Click);
            ingridMenuItem.Click += new EventHandler(ingridMenuItem_Click);

            newButton.Click += new EventHandler(newMenuItem_Click);
            openButton.Click += new EventHandler(openMenuItem_Click);
            saveButton.Click += new EventHandler(saveMenuItem_Click);
            executeButton.Click += new EventHandler(executeMenuItem_Click);
            cancelexecuteButton.Click += new EventHandler(cancelexecuteMenuItem_Click);

            cascadeMenuItem.Click += new EventHandler(cascadeMenuItem_Click);
            htileMenuItem.Click += new EventHandler(htileMenuItem_Click);
            vtileMenuItem.Click += new EventHandler(vtileMenuItem_Click);
            ResetGuiState();

            browerManager = new BrowserManager(tvwBrowser);
            browerManager.HelpTextProduced =
                text => { if (null != CurrentExecutor) CurrentExecutor.RelatedView.RawQuerySqls = text; };

            cboDatabaseList.DataBindings.DefaultDataSourceUpdateMode = DataSourceUpdateMode.OnPropertyChanged;
            cboDatabaseList.DataBindings.Add("Text", browerManager, "BrowsingDatabase");

            connectMenuItem.PerformClick();
        }

        private void connectMenuItem_Click(object sender, EventArgs e)
        {
            
            IAFMConnection newConnection = 
                Settings.Default.IsLocalSql ? 
                ConnectionManager.BuildConnection("Initial Catalog=master;Data Source=localhost;Integrated Security=SSPI;") : 
                ConnectionManager.BuildConnection(new ConnectForm());
            if (newConnection == null)
            {
                if (!ConnectionManager.UserCanceledConnection)
                    MessageBox.Show("对不起, 连接远端服务器失败!");
            }
            else
            {
                currentConnection = newConnection;                
                QueryHelper.Command = Settings.Default.IsLocalSql ?
                    (IAFMCommand) new SqlCommandWrapper(null, currentConnection) :
                    (IAFMCommand) new WCFCommand(null, currentConnection);
                
                DatabaseList = QueryHelper.RetrieveDatabaseList();
                browerManager.BrowsingDatabase = QueryHelper.CurrentDbInfo.DbName;
                browerManager.Initialize();
                newMenuItem_Click(sender, e);
            }
        }

        private void newMenuItem_Click(object sender, EventArgs e)
        {
            QueryExecutor queryExecutor = 
                new QueryExecutor(new QueryForm(this, QueryTextFile.NewTextFile()), 
                    Settings.Default.IsLocalSql ?
                    (IAFMCommand) new SqlCommandWrapper(null, currentConnection) :
                    (IAFMCommand) new WCFCommand(null, currentConnection));            
            queryExecutor.ReadyForInput();
            queryExecutorList.Add(queryExecutor);
            ResetGuiState();
        }

        private void openMenuItem_Click(object sender, EventArgs e)
        {
            //bad smells
            QueryForm queryForm = ActiveMdiChild as QueryForm;
            queryForm.OpenFile();
        }                      

        private void saveMenuItem_Click(object sender, EventArgs e)
        {
            //bad smells;
            QueryForm queryForm = ActiveMdiChild as QueryForm;
            queryForm.SaveFile();
        }

        private void saveasMenuItem_Click(object sender, EventArgs e)
        {
            //bad smells
            QueryForm queryForm = ActiveMdiChild as QueryForm;
            queryForm.SaveFileAs();
        }

        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            //TODO::: some cleanup
            Close();
        }

        private void executeMenuItem_Click(object sender, EventArgs e)
        {
            if (CurrentExecutor != null)
                CurrentExecutor.ExecuteQueryAsnc();        
        }
        
        private void cancelexecuteMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Sorry, Not implemented yet!");
        }

        private void ingridMenuItem_Click(object sender, EventArgs e)
        {
            QueryExecutor.ResultInText = !QueryExecutor.ResultInText;
            intextMenuItem.Checked = QueryExecutor.ResultInText;
            ingridMenuItem.Checked = !QueryExecutor.ResultInText;
        }

        private void intextMenuItem_Click(object sender, EventArgs e)
        {
            QueryExecutor.ResultInText = !QueryExecutor.ResultInText;
            intextMenuItem.Checked = QueryExecutor.ResultInText;
            ingridMenuItem.Checked = !QueryExecutor.ResultInText;
        }

        void cascadeMenuItem_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.Cascade);
        }

        void htileMenuItem_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileHorizontal);
        }

        void vtileMenuItem_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileVertical);
        }

        void aboutMenuItem_Click(object sender, EventArgs e)
        {
            (new AboutForm()).ShowDialog();
        }

        private QueryExecutor CurrentExecutor
        {
            get {
                QueryForm queryForm = ActiveMdiChild as QueryForm;
                if (queryForm == null) return null;
                foreach (QueryExecutor queryExecutor in queryExecutorList)
                {
                    if (Object.ReferenceEquals(queryExecutor.RelatedView, queryForm))
                    {
                        return queryExecutor;
                    }
                }
                return null;
            }
        }

        private void MainForm_MdiChildActivate(object sender, EventArgs e)
        {
            this.ResetGuiState();
        }

        internal void ResetGuiState()
        {
            QueryExecutor executor = CurrentExecutor;
            bool hasExecutor = (executor != null);

            newMenuItem.Enabled = newButton.Enabled =
                saveMenuItem.Enabled = saveButton.Enabled =
                saveasMenuItem.Enabled = hasExecutor;
            openMenuItem.Enabled = openButton.Enabled = hasExecutor && !executor.IsExecuting;

            executeMenuItem.Enabled = executeButton.Enabled = hasExecutor && !executor.IsExecuting;
            cancelexecuteMenuItem.Enabled = cancelexecuteButton.Enabled = hasExecutor && executor.IsExecuting;

            intextMenuItem.Enabled = ingridMenuItem.Enabled = hasExecutor;

            windowMenuItem.Visible = hasExecutor;

            panelBrowser.Visible = hasExecutor;
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            foreach (QueryExecutor executor in queryExecutorList)
            {
               executor.Close();
            }
            QueryHelper.Command = null; //i think will dispose
            currentConnection = null;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }
    }
}
