﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Threading;
using System.IO;
using System.Diagnostics;

using AnswerForMe.Model;

namespace AnswerForMe.Presentation
{
    internal class QueryExecutor
    {
        private IQueryExecutorView view;
        private IAFMCommand command;
        private Thread workerThread;

        private bool isExecuting;

        internal static bool ResultInText { get; set; }

        // view not null, command not null and has connected and opened
        internal QueryExecutor(IQueryExecutorView view, IAFMCommand command) 
        {
            this.view = view;
            this.command = command;
        }

        private String rawQuerySqls;

        private String[] QuerySqlList
        {
            get
            {
                return rawQuerySqls.Split(new String[] { "go", "GO", "Go", "gO" }, StringSplitOptions.RemoveEmptyEntries);
            }
        }

        internal IQueryExecutorView RelatedView
        {
            get
            {
                return view;
            }
        }

        internal bool IsExecuting 
        {
            get
            {
                return isExecuting;
            }
            set
            {
                isExecuting = value;
            }
        }
      
        internal void ReadyForInput()
        {
            view.DbInfoRetrieved = QueryHelper.CurrentDbInfo;
            view.ReadyForInput();
        }

        internal void ExecuteQueryAsnc()
        {
            rawQuerySqls = view.RawQuerySqls;
            workerThread = new Thread(new ThreadStart(ExecuteQuery));
            workerThread.Name = "QueryExecutor WorkerThread";
            workerThread.IsBackground = true;
            workerThread.Start();
            //ExecuteQuery();
        }

        internal void CancelQueryAsnc()
        {
            //if (workerThread != null)
            //{
            //    if (workerThread.IsAlive) workerThread.Abort();
            //    IsExecuting = false;
            //    workerThread = null;
            //}
        }

        internal void ExecuteQuery()
        {
            DateTime startTime = DateTime.Now;
            DataSet dataSet = null;           
            int total = 0;

            IsExecuting = true;
            if (view.QueryStartedAction != null)
                view.Invoke(view.QueryStartedAction);
            try
            {
                QueryResult queryResult = new QueryResult { ResultDataSet = new DataSet(), ResultText = null };
                foreach (String s in QuerySqlList)
                {
                    command.CommandText = s;
                    Trace.WriteLine(s);
                    dataSet = command.ExecuteQuery();
                    if (dataSet != null)
                    {
                        DataTable table;
                        while (dataSet.Tables.Count > 0)
                        {
                            total++;
                            table = dataSet.Tables[0];
                            dataSet.Tables.Remove(table);
                            table.TableName = "结果" + (total == 1 ? "" : total.ToString());
                            queryResult.ResultDataSet.Tables.Add(table);
                        }
                    }
                }
                if (ResultInText) queryResult.WriteTextResult();
                queryResult.ExcuteTimeCost = DateTime.Now.Subtract(startTime);
                view.Result = queryResult;
                IsExecuting = false;
                if (view.QuerySuccessAction != null)
                    view.Invoke(view.QuerySuccessAction);
            }
            catch
            {
                IsExecuting = false;
                view.Result = null;
                if (view.QueryFailedAction != null)
                    view.Invoke(view.QueryFailedAction);
            }
        }

        internal void Close()
        {
            try
            {
                if (workerThread != null)
                    if (workerThread.IsAlive)
                        workerThread.Join();
            }
            catch { }
            view = null;
            command.Dispose();
            command = null;
        }
    }
}
