﻿using System;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;

namespace AnswerForMe.Presentation
{
    public partial class QueryForm : Form, IQueryExecutorView
    {
        private QueryTextFile qtf;

        public QueryForm(Form mdiParent, QueryTextFile qtf)
        {
            InitializeComponent();
            this.MdiParent = mdiParent;
            this.qtf = qtf;
            this.txtRawQuerySql.DataBindings.Add("Text", qtf, "CurrentText");
        }

        private String Title
        {
            get
            {
                return String.Format("查询 - {0}", new[] {this.qtf.TextFileName}); 
            }
        }

        public String RawQuerySqls
        {
            get { return txtRawQuerySql.Text; }
            set { txtRawQuerySql.Text = value; }
        }

        public QueryResult Result { get; set; }

        public Action QueryStartedAction
        {
            get { return new Action(QueryStarted); }
        }

        public Action QuerySuccessAction
        {
            get { return new Action(QuerySuccess); } 
        }

        public Action QueryFailedAction
        {
            get { return new Action(QueryFailed); } 
        }

        public void OpenFile()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() != DialogResult.OK)
                return;
            this.qtf = QueryTextFile.OpenTextFile(openFileDialog.FileName);
            this.Text = this.Title;
        }

        public void SaveFile()
        {
            if (qtf.IsNewFile)
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();
                saveFileDialog.ShowDialog();
                if (String.IsNullOrEmpty(saveFileDialog.FileName)) return;
                qtf.TextFilePath = saveFileDialog.FileName;
            }
            qtf.SaveFile();
            this.Text = this.Title;
        }

        public void SaveFileAs()
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.FileName = qtf.IsNewFile ? qtf.TextFileName : qtf.TextFilePath;
            saveFileDialog.ShowDialog();
            if (String.IsNullOrEmpty(saveFileDialog.FileName)) return;
            qtf.TextFilePath = saveFileDialog.FileName;
            qtf.SaveFile();
            this.Text = this.Title;
        }

            
        public void ReadyForInput()
        {
            this.Show();
        }

        private void QueryStarted()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
        }

        private void QueryFailed()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
        }

        private void QuerySuccess()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
            
            this.tabResult.TabPages.Clear();

            if (Result == null) return;

            DataSet ds = Result.ResultDataSet;
            if (ds == null || ds.Tables.Count == 0) return;

            if (Result.ResultText == null)
            {
                const int MaxTables = 20;
                for (int i = 0; i < Math.Min(ds.Tables.Count, MaxTables); i++)
                    CreateGridPage(ds.Tables[i]);
            }
            else
                CreateTextPage(Result.ResultText);
        }

        private void CreateGridPage(DataTable dt)
        {
            DataGridView dgv = new DataGridView();
            dgv.Dock = DockStyle.Fill;
            dgv.BorderStyle = BorderStyle.Fixed3D;
            dgv.ReadOnly = true;
            dgv.AllowUserToAddRows = false;
            dgv.AllowUserToDeleteRows = false;
            dgv.AutoGenerateColumns = true;
            TabPage page = new TabPage(dt.TableName);
            page.Controls.Add(dgv);
            tabResult.TabPages.Add(page);
            //Must be here
            dgv.DataSource = dt;
            dgv.AutoResizeColumns();
        }

        private void CreateTextPage(String s)
        {
            RichTextBox rtf = new RichTextBox();
            rtf.AutoSize = false;
            rtf.Dock = DockStyle.Fill;
            rtf.Multiline = true;
            rtf.WordWrap = false;
            rtf.ScrollBars = RichTextBoxScrollBars.Both;
            rtf.MaxLength = 0;
            rtf.AppendText(s);

            TabPage page = new TabPage("结果");
            page.Controls.Add(rtf);

            tabResult.TabPages.Add(page);
        }
    }
}
