﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Diagnostics;

using AnswerForMe.Model;

namespace AnswerForMe.Presentation
{
    class QueryHelper
    {
        private static IAFMCommand command;
        private static readonly string retrievDbInfoSql = "Select @@ServerName, ISNULL(SUSER_SNAME(), SUSER_NAME()), Db_Name()";
        private static readonly Dictionary<string, string> browerObjectListSql
            = new Dictionary<string, string>()
            {
                {"用户表", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'U' ORDER BY objowner, objname"},
                {"系统表", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'S' ORDER BY objowner, objname"},
                {"视图", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'V' ORDER BY objowner, objname"},
                {"用户存储过程", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'P' AND ObjectProperty(id, N'IsMSShipped') = 0 ORDER BY objowner, objname"},
                {"系统存储过程", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'P' AND ObjectProperty(id, N'IsMSShipped') = 1 ORDER BY objowner, objname"},
                {"扩展过程", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'X' ORDER BY objowner, objname"},
                {"函数", @"SELECT id, name objname, user_name(uid) objowner FROM {0}sysobjects 
                    WHERE xtype = N'FN' ORDER BY objowner, objname"},
            };
        private static string browserColumnListSqlTmpl =
            "SELECT COL.NAME COLUMN_NAME, TYP.NAME DATA_TYPE, (CASE WHEN COL.LENGTH IS NOT NULL THEN '(' + RTRIM(COL.LENGTH)+')'" + 
            "WHEN COL.PREC IS NOT NULL THEN '('+RTRIM(COL.PREC)+','+RTRIM(COL.SCALE)+')' ELSE '' END) EXTENSION, " +
            "(CASE COL.ISNULLABLE WHEN 1 THEN 'YES' ELSE 'NO ' END) IS_NULLABLE " + 
            "FROM {1}SYSCOLUMNS COL, {1}SYSTYPES TYP " + 
            "WHERE COL.XUSERTYPE = TYP.XUSERTYPE AND COL.ID = {0} ORDER BY COL.COLORDER";
            //"SELECT COLUMN_NAME,DATA_TYPE,(CASE WHEN CHARACTER_MAXIMUM_LENGTH IS NOT NULL THEN '('+RTRIM(CHARACTER_MAXIMUM_LENGTH)+')'" +
            //"WHEN NUMERIC_PRECISION IS NOT NULL THEN '('+RTRIM(NUMERIC_PRECISION)+','+RTRIM(NUMERIC_SCALE)+')' ELSE '' END) EXTENSION," + 
            //"IS_NULLABLE FROM {2}INFORMATION_SCHEMA.COLUMNS " + 
            //"WHERE TABLE_CATALOG = DB_NAME() AND TABLE_SCHEMA = N'{0}' AND TABLE_NAME = N'{1}' ORDER BY ORDINAL_POSITION";

        private static string helptextSqlTmpl = 
        @"declare @BlankSpaceAdded int, @BasePos int, @CurrentPos int, @TextLength int, @LineId int, @AddOnLen int, @LFCR int, @DefinedLength int,@SyscomText nvarchar(4000),@Line nvarchar(255) 
        Select @DefinedLength = 255 
        SELECT @BlankSpaceAdded = 0 
        CREATE TABLE #CommentText (LineId int, Text nvarchar(255) collate database_default)
        DECLARE ms_crs_syscom CURSOR LOCAL
        FOR SELECT text FROM {1}syscomments WHERE id = {0} and encrypted = 0
        ORDER BY number, colid
        FOR READ ONLY 
        SELECT @LFCR = 2
        SELECT @LineId = 1
        OPEN ms_crs_syscom
        FETCH NEXT FROM ms_crs_syscom into @SyscomText
        WHILE @@fetch_status >= 0
        BEGIN
            SELECT  @BasePos    = 1
            SELECT  @CurrentPos = 1
            SELECT  @TextLength = LEN(@SyscomText)

            WHILE @CurrentPos  != 0
            BEGIN
                SELECT @CurrentPos =   CHARINDEX(char(13)+char(10), @SyscomText, @BasePos)
                IF @CurrentPos != 0
                BEGIN
                    While (isnull(LEN(@Line),0) + @BlankSpaceAdded + @CurrentPos-@BasePos + @LFCR) > @DefinedLength
                    BEGIN
                        SELECT @AddOnLen = @DefinedLength-(isnull(LEN(@Line),0) + @BlankSpaceAdded)
                        INSERT #CommentText VALUES
                        ( @LineId,
                          isnull(@Line, N'') + isnull(SUBSTRING(@SyscomText, @BasePos, @AddOnLen), N''))
                        SELECT @Line = NULL, @LineId = @LineId + 1,
                               @BasePos = @BasePos + @AddOnLen, @BlankSpaceAdded = 0
                    END
                    SELECT @Line    = isnull(@Line, N'') + isnull(SUBSTRING(@SyscomText, @BasePos, @CurrentPos-@BasePos + @LFCR), N'')
                    SELECT @BasePos = @CurrentPos+2
                    INSERT #CommentText VALUES( @LineId, @Line )
                    SELECT @LineId = @LineId + 1
                    SELECT @Line = NULL
                END
                ELSE
                BEGIN
                    IF @BasePos <= @TextLength
                    BEGIN
                        While (isnull(LEN(@Line),0) + @BlankSpaceAdded + @TextLength-@BasePos+1 ) > @DefinedLength
                        BEGIN
                            SELECT @AddOnLen = @DefinedLength - (isnull(LEN(@Line),0) + @BlankSpaceAdded)
                            INSERT #CommentText VALUES
                            ( @LineId,
                              isnull(@Line, N'') + isnull(SUBSTRING(@SyscomText, @BasePos, @AddOnLen), N''))
                            SELECT @Line = NULL, @LineId = @LineId + 1,
                                @BasePos = @BasePos + @AddOnLen, @BlankSpaceAdded = 0
                        END
                        SELECT @Line = isnull(@Line, N'') + isnull(SUBSTRING(@SyscomText, @BasePos, @TextLength-@BasePos+1 ), N'')
                        if LEN(@Line) < @DefinedLength and charindex(' ', @SyscomText, @TextLength+1 ) > 0
                        BEGIN
                            SELECT @Line = @Line + ' ', @BlankSpaceAdded = 1
                        END
                    END
                END
            END
	        FETCH NEXT FROM ms_crs_syscom into @SyscomText
        END

        IF @Line is NOT NULL
            INSERT #CommentText VALUES( @LineId, @Line )
        select Text from #CommentText order by LineId
        CLOSE  ms_crs_syscom
        DEALLOCATE 	ms_crs_syscom
        DROP TABLE 	#CommentText";

        private static DbInfo currentDbInfo;

        public static IAFMCommand Command
        {
            get { return command; }
            set
            {
                command = value;
                currentDbInfo = new DbInfo();
                if (command != null)
                {
                    command.CommandText = retrievDbInfoSql;
                    DataSet ds = command.ExecuteQuery();
                    if (ds != null)
                    {
                        DataRow row = ds.Tables[0].Rows[0];
                        currentDbInfo.ServerName = row[0].ToString();
                        currentDbInfo.LoingName = row[1].ToString();
                        currentDbInfo.DbName = row[2].ToString();
                    }
                }
            }
        }

        public static DbInfo CurrentDbInfo
        {
            get { return currentDbInfo; }
        }

        public static DataTable RetrieveObjectList(string typeName, string database = null)
        {
            Command.CommandText = string.Format(browerObjectListSql[typeName], database == null ? "" : database + "..");
            DataSet ds = Command.ExecuteQuery();
            if (ds == null) return null;
            if (ds.Tables.Count == 0) return null;
            return ds.Tables[0];
        }

        public static DataTable RetrieveColumnList(int id, string database = null)
        {
            Command.CommandText = string.Format(browserColumnListSqlTmpl, id, database == null ? "" : database + "..");
            Trace.WriteLine(Command.CommandText);
            DataSet ds = Command.ExecuteQuery();
            if (ds == null) return null;
            if (ds.Tables.Count == 0) return null;
            return ds.Tables[0];
        }

        public static string[] RetrieveDatabaseList()
        {
            Command.CommandText = "SELECT name FROM master..sysdatabases ORDER BY name";
            DataSet ds = Command.ExecuteQuery();
            if (ds == null) return null;
            if (ds.Tables.Count == 0) return null;
            DataRowCollection rows = ds.Tables[0].Rows;
			string[] result = new string [rows.Count];
			int count = 0;
			foreach (DataRow row in rows)
				result [count++] = row[0].ToString().Trim();
			return result;
        }

        public static string RetrieveHelpText(int id, string database = null)
        {
            Command.CommandText = string.Format(helptextSqlTmpl, id, database == null ? "" : database + "..");
            DataSet ds = Command.ExecuteQuery();
            if (ds == null) return null;
            if (ds.Tables.Count == 0) return null;
            StringBuilder sb = new StringBuilder();
            bool altered = false;
            foreach (DataRow row in ds.Tables[0].Rows)
            {
                string line = row[0].ToString();
                if (!altered && line.Trim().ToUpper().StartsWith("CREATE"))
                {
                    sb.Append("ALTER" + line.Trim().Substring(6, line.Trim().Length - 6) + "\r\n");
                    altered = true;
                }
                else
                    sb.Append(line);
            }
            return sb.ToString().Trim();
        }

        public static bool IsDataTypeFixed(string datatype)
        {
            return !(datatype == "decimal" || datatype == "numeric" || datatype == "binary" || datatype == "varbinary"
                || datatype == "char" || datatype == "nchar" || datatype == "varchar" || datatype == "nvarchar");
        }
    }
}
