﻿using System;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;

namespace AnswerForMe.Presentation
{
    public partial class QueryForm : Form, IQueryExecutorView
    {
        private QueryTextFile qtf;
        private DateTime ExecuteStartedTime;
        private static string fileFilterList = "查询文件(*.sql)|*.sql|文本文件(*.txt)|*.txt|所有文件(*.*)|*.*";
        public QueryForm(Form mdiParent, QueryTextFile qtf)
        {
            InitializeComponent();
            this.MdiParent = mdiParent;
            this.qtf = qtf;
            this.Text = this.Title;
        }

        private String Title
        {
            get
            {
                return String.Format("查询 - {0}", this.qtf.TextFileName); 
            }
        }

        public DbInfo DbInfoRetrieved
        {
            set 
            {
                lblServerName.Text = value.ServerName;
                lblLoginName.Text = value.LoingName;
                lblDatabase.Text = value.DbName;
            }
        }
        public String RawQuerySqls
        {
            get 
            {
                if (txtRawQuerySql.SelectionLength == 0)
                    return txtRawQuerySql.Text;
                else
                    return txtRawQuerySql.SelectedText;
            }
            set 
            {
                if (txtRawQuerySql.SelectionLength == 0)
                    txtRawQuerySql.Text = value;
                else
                    txtRawQuerySql.SelectedText = value;
            }
        }

        public QueryResult Result { get; set; }

        public Action QueryStartedAction
        {
            get { return new Action(QueryStarted); }
        }

        public Action QuerySuccessAction
        {
            get { return new Action(QuerySuccess); } 
        }

        public Action QueryFailedAction
        {
            get { return new Action(QueryFailed); } 
        }

        public void OpenFile()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = fileFilterList;
            if (openFileDialog.ShowDialog() != DialogResult.OK)
                return;
            qtf = QueryTextFile.OpenTextFile(openFileDialog.FileName);
            txtRawQuerySql.Text = qtf.CurrentText;
            this.Text = this.Title;
        }

        public void SaveFile()
        {
            if (qtf.IsNewFile)
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();
                saveFileDialog.Filter = fileFilterList;
                saveFileDialog.ShowDialog();
                if (String.IsNullOrEmpty(saveFileDialog.FileName)) return;
                qtf.TextFilePath = saveFileDialog.FileName;
            }
            qtf.CurrentText = txtRawQuerySql.Text;
            qtf.SaveFile();
            this.Text = this.Title;
        }

        public void SaveFileAs()
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = fileFilterList;
            saveFileDialog.FileName = qtf.IsNewFile ? qtf.TextFileName : qtf.TextFilePath;
            saveFileDialog.ShowDialog();
            if (String.IsNullOrEmpty(saveFileDialog.FileName)) return;
            qtf.TextFilePath = saveFileDialog.FileName;
            qtf.CurrentText = txtRawQuerySql.Text;
            qtf.SaveFile();
            this.Text = this.Title;
        }

            
        public void ReadyForInput()
        {
            this.Show();
        }

        private void QueryStarted()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
            lblExecuteStatus.Text = "正在执行查询......";
            ExecuteStartedTime = DateTime.Now;
            lblExecuteTime.Text = "0:00:00";
            this.timerExcuting.Start();
        }

        private void QueryFailed()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
            lblExecuteStatus.Text = "抱歉,执行查询失败!";
            this.timerExcuting.Stop();
        }

        private void QuerySuccess()
        {
            MainForm parent = this.MdiParent as MainForm;
            parent.ResetGuiState();
            lblExecuteStatus.Text = "执行查询成功.";
            this.timerExcuting.Stop();
            
            this.tabResult.TabPages.Clear();

            if (Result == null) return;

            DataSet ds = Result.ResultDataSet;
            if (ds == null || ds.Tables.Count == 0) return;

            const int MaxTables = 20;
            for (int i = 0; i < Math.Min(ds.Tables.Count, MaxTables); i++)
            {
                if (Result.ResultText == null)
                    CreateGridPage(ds.Tables[i]);
                else
                    CreateTextPage(Result.ResultText[i], i);
            }
        }

        private void CreateGridPage(DataTable dt)
        {
            DataGridView dgv = new DataGridView();
            dgv.Dock = DockStyle.Fill;
            dgv.BorderStyle = BorderStyle.Fixed3D;
            dgv.ReadOnly = true;
            dgv.AllowUserToAddRows = false;
            dgv.AllowUserToDeleteRows = false;
            dgv.AutoGenerateColumns = true;
            TabPage page = new TabPage(dt.TableName);
            page.Controls.Add(dgv);
            tabResult.TabPages.Add(page);
            //Must be here
            dgv.DataSource = dt;
            dgv.AutoResizeColumns();
        }

        private void CreateTextPage(String s, int index)
        {
            RichTextBox rtf = new RichTextBox();
            rtf.AutoSize = false;
            rtf.Dock = DockStyle.Fill;
            rtf.Multiline = true;
            rtf.WordWrap = false;
            rtf.ScrollBars = RichTextBoxScrollBars.Both;
            rtf.MaxLength = 0;
            rtf.AppendText(s);

            TabPage page = new TabPage("结果"+(index == 0 ? "" : index.ToString()));
            page.Controls.Add(rtf);

            tabResult.TabPages.Add(page);
        }

        private void timerExcuting_Tick(object sender, EventArgs e)
        {
            TimeSpan eplapsed = DateTime.Now -ExecuteStartedTime;
            lblExecuteTime.Text = 
                string.Format("{0:00}:{1:00}:{2:00}", eplapsed.Hours, eplapsed.Minutes, eplapsed.Seconds);
        }
    }
}
