﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

using AnswerForMe.Model;

namespace AnswerForMe.Presentation
{
    enum WCFDbNodeType 
    {
        UserTableList,
        SystemTableList,
        ViewList,
        UserStoredProcList,
        SystemStoredProcList,
        FunctionList,
        
        UserTable,
        SystemTable,
        View,
        UserStoredProc,
        SystemStoredProc,
        Function,

        ColumnList,
        Column, 

        NULLType
    };

    class WCFDbNode
    {
        //string RenderType; //U 用户表, S 系统表, V 视图, P 存储过程, FN 函数
        WCFDbNodeType type;
        string name;
        string owner = "";
        string safeName;
        string sqlForChildren;
        WCFDbNode parent;
        private static string objSqlTemplate =
            "select object_name(id) objname, user_name(uid) objowner from sysobjects where {0} order by objowner, objname";
        private static string colSqlTemplate = 
            "SELECT COLUMN_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH, NUMERIC_PRECISION, NUMERIC_SCALE, IS_NULLABLE" + 
            " FROM INFORMATION_SCHEMA.COLUMNS " + 
            " WHERE TABLE_CATALOG = DB_NAME() AND TABLE_SCHEMA = N'{0}' AND TABLE_NAME = N'{1}' ORDER BY ORDINAL_POSITION";
        public static WCFCommand Command;
        public static WCFDbNode WCFUserTablesNode = new WCFDbNode()
        {
            type = WCFDbNodeType.UserTableList,
            name = "用户表",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'U'")
        };
        public static WCFDbNode WCFSystemTablesNode = new WCFDbNode()
        {
            type = WCFDbNodeType.SystemTableList,
            name = "系统表",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'S'")
        };
        public static WCFDbNode WCFViewsNode = new WCFDbNode()
        {
            type = WCFDbNodeType.ViewList,
            name = "视图",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'V'")
        };
        public static WCFDbNode WCFUserStoredProcsNode = new WCFDbNode()
        {
            type = WCFDbNodeType.UserStoredProcList,
            name = "用户存储过程",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'P' and ObjectProperty (id, N'IsMSShipped') = 0")
        };
        public static WCFDbNode WCFSystemStoreProcsNode = new WCFDbNode()
        {
            type = WCFDbNodeType.SystemStoredProcList,
            name = "系统存储过程",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'P' and ObjectProperty (id, N'IsMSShipped') <> 0")
        };
        public static WCFDbNode WCFFunctionsNode = new WCFDbNode()
        {
            type = WCFDbNodeType.FunctionList,
            name = "函数",
            sqlForChildren = string.Format(objSqlTemplate, "xtype = N'FN'")
        };
        public static WCFDbNode[] WCFDbRootNodes = new WCFDbNode[] 
        {
            WCFUserTablesNode,
            WCFSystemTablesNode,
            WCFViewsNode,
            WCFUserStoredProcsNode,
            WCFSystemStoreProcsNode,
            WCFFunctionsNode
        };

        public WCFDbNodeType NodeType
        {
            get { return type; }
        }
        public string NodeName
        {
            get { return name; }
        }
        public string NodeOwner
        {
            get { return owner; }
        }
        public string NodeSafeName
        {
            get { return safeName; }
        }
        public WCFDbNode NodeParent
        {
            get { return parent; }
        }

        public WCFDbNode[] GetChildNodes()
        {
            if (Array.IndexOf<WCFDbNode>(WCFDbRootNodes, this) != -1)
                return GetRootsChildNodes();
            else if (this.type == WCFDbNodeType.UserTable || this.type == WCFDbNodeType.View)
                return new WCFDbNode[]{
                    new WCFDbNode() {
                        type = WCFDbNodeType.ColumnList,
                        name = "列",
                        parent = this
                    }
                };
            else if (this.type == WCFDbNodeType.ColumnList)
                return GetColumns();
            else
                return null;
        }

        public bool IsLeafNode
        {
            get
            {   return NodeType == WCFDbNodeType.SystemTable ||
                    NodeType == WCFDbNodeType.UserStoredProc ||
                    NodeType == WCFDbNodeType.SystemStoredProc ||
                    NodeType == WCFDbNodeType.Function ||
                    NodeType == WCFDbNodeType.Column;
            }
        }

        private WCFDbNode[] GetColumns()
        {
            string tableowner = parent.owner;
            string tablename = parent.name;
            Command.CommandText = string.Format(colSqlTemplate, tableowner, tablename);
            DataSet ds = Command.ExecuteQuery();
            if (ds == null || ds.Tables.Count == 0) return new WCFDbNode[]{};

            string typeExtend;
            string nullable;
            List<WCFDbNode> list = new List<WCFDbNode>();

            //"SELECT COLUMN_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH, NUMERIC_PRECISION, NUMERIC_SCALE, IS_NULLABLE" + 
            //" FROM INFORMATION_SCHEMA.COLUMNS " + 
            //" WHERE TABLE_CATALOG = DB_NAME() AND TABLE_SCHEMA = N'{0}' AND TABLE_NAME = N'{1}' ORDER BY ORDINAL_POSITION";
            foreach (DataRow row in ds.Tables[0].Rows)
            {
                if (row["CHARACTER_MAXIMUM_LENGTH"].ToString() != "")
                    typeExtend = "(" + row["CHARACTER_MAXIMUM_LENGTH"].ToString() + ")";
                else if (row["NUMERIC_PRECISION"].ToString() != "")
                    typeExtend = "(" + row["NUMERIC_PRECISION"].ToString() + "," + row["NUMERIC_SCALE"].ToString() + ")";
                else
                    typeExtend = "";
                nullable = row["IS_NULLABLE"].ToString().StartsWith("Y") ? "Null" : "Not Null";
                list.Add(new WCFDbNode()
                {
                    type = WCFDbNodeType.Column,
                    name = row["COLUMN_NAME"].ToString() + " " + "(" +
                        row["DATA_TYPE"].ToString() + typeExtend + "," + " " + nullable + ")",
                    parent = this
                });
            }
            return list.ToArray<WCFDbNode>();
        }

        private WCFDbNode[] GetRootsChildNodes()
        {
            Command.CommandText = sqlForChildren;
            DataSet ds = Command.ExecuteQuery();
            if (ds == null || ds.Tables.Count == 0) return new WCFDbNode[]{};

            List<WCFDbNode> list = new List<WCFDbNode>();
            WCFDbNodeType childNodeType = ChildNodeType;
            foreach (DataRow row in ds.Tables[0].Rows)
            {
                list.Add(new WCFDbNode()
                {
                    type = childNodeType,
                    name = row["objname"].ToString(),
                    owner = row["objowner"].ToString(),
                    parent = this
                });
            }
            return list.ToArray<WCFDbNode>();
        }

        private WCFDbNodeType ChildNodeType
        {
            get
            {
                switch (type)
                {
                    case WCFDbNodeType.UserTableList:
                        return WCFDbNodeType.UserTable;
                    case WCFDbNodeType.SystemTableList:
                        return WCFDbNodeType.SystemTable;
                    case WCFDbNodeType.ViewList:
                        return WCFDbNodeType.View;
                    case WCFDbNodeType.UserStoredProcList:
                        return WCFDbNodeType.UserStoredProc;
                    case WCFDbNodeType.SystemStoredProcList:
                        return WCFDbNodeType.SystemStoredProc;
                    case WCFDbNodeType.FunctionList:
                        return WCFDbNodeType.Function;
                    case WCFDbNodeType.ColumnList:
                        return WCFDbNodeType.Column;
                    default:
                        return WCFDbNodeType.NULLType;
                }
            }
        }
    }
}